//$Header: /cvsroot/esrg/sfesrg/esrgnxpj/sfnthcgi0304/arith_large_cgi.c,v 1.6 2003/07/01 03:46:58 dtashley Exp $
//********************************************************************************
//This source file is the main function of a program to support number theory
//web pages.
//********************************************************************************
//Copyright (C) 2003 David T. Ashley
//********************************************************************************
//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//This program or source file is distributed in the hope that it will 
//be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You may have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//********************************************************************************
//This program is written exclusively to support a PHP script which must calculate and
//display a number of number theory results.  The necessity of this script
//(rather than using BCMATH) comes about because SourceForge does not have
//bcmath installed on its PHP system.
//
//This program is meant to be called by the PHP script using the exec()
//call, but could also be run by hand (and in fact this method is used for testing).
//
//The first parameter after the program name is the suboperation to be performed
//(each CGI page probably uses a different suboperation).  This program
//will return a non-zero exit code only if there is a missing or unrecognized
//suboperation.  Otherwise, error reporting is controlled by the rules of the
//suboperation.
//
//The currently defined suboperations are:
//
//   a)"help"
//     Display information about what the program is and how to use
//     it.
//   b)"gcd"
//     Calculate the gcd of two integers using Euclid's classic
//     algorithm and also display the intermediate results.
//   c)"gmp_prob_prime"
//     Use Miller-Rabin to determine to high certainty whther a 
//     number is prime or composite.
//   d)"pfact_18"     
//     Attempt to factor a number of 18 decimal digits or less into     
//     component primes.
//   e)"cfbrap"
//     Finds the best rational approximation to a rational number subject
//     to constraints on the denominator and numerator using continued
//     fraction techniques.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "auxfuncs.h"
#include "subfunc_cfbrap.h"
#include "subfunc_gcd.h"
#include "subfunc_gmp_prob_prime.h"
#include "subfunc_pfact_18.h"

//Single function prototype needed for a forward reference.
static void dump_subfunction_choices(int indent);

//Implements the help.
int SUBFUNC_HELP_main(int argc, char *argv[])
   {
     int i;
   char *help[] =
     {
     "This program is a compiled 'C'-language program designed to be called from a",
     "PHP script to assist in calculating number theory and large integer",
     "arithmetic results.  This scheme (of having a separate executable like this",
     "one) was used with SourceForge web content because SourceForge did not have",
     "the bcmath library compiled into PHP, and it was determined experimentally",
     "that writing large integer arithmetic functions in PHP directly resulted in",
     "unusably slow performance.  This program makes use of the GMP library, which",
     "gives it superior performance.",
     "",
     "This program can of course be invoked from a shell or from another scripting",
     "language besides PHP.  It is designed, however, for execution from a",
     "scripting language, as you might guess from the human-unfriendly output.",
     "",
     "Please contact Dave Ashley (dtashley@aol.com) with any questions or",
     "concerns.",
     "",
     "Dave Ashley, Detroit, Michigan, USA, April, 2003.",
     "",
     "The available subfunction choices (the first parameter on the command",
     "line) are:",
     };

   for (i=0; i<sizeof(help)/sizeof(help[0]); i++)
     printf("%s\n", help[i]);
   dump_subfunction_choices(3);
   }

//Structure type used to hold the jump table of different functions
//to handle different subcommands.
struct struct_ARITH_LARGE_CGI_subcmd_jmp 
      {
      char *subfunc_string;
      int (*subfunc_ptr)(int argc, char *argv[]);
      };

//The jump table.  The last element is defined to have both a
//NULL string pointer and a NULL function pointer.
static struct struct_ARITH_LARGE_CGI_subcmd_jmp subcmd_jump_tbl[] =
      {
      {"cfbrap",            SUBFUNC_CFBRAP_main               },
      {"gcd",               SUBFUNC_GCD_main                  },
      {"gmp_prob_prime",    SUBFUNC_GMP_PROB_PRIME_main       },
      {"pfact_18",          SUBFUNC_PFACT_18_main             },
      {"help",              SUBFUNC_HELP_main                 },
      {NULL,                NULL                              }
      };

//Dumps the available choices for the subfunction code to the
//standard output, each indented by "indent" characters.
static void dump_subfunction_choices(int indent)
   {
   int i, j;

   for (i=0; subcmd_jump_tbl[i].subfunc_string != NULL; i++)
      {
      for (j=0; j<indent; j++)
         printf(" ");
      printf("%s\n", subcmd_jump_tbl[i].subfunc_string);
      }
   }


int main(int argc, char *argv[])
   {   
   int i;
   int rv;

   if (argc < 2)
      {
      printf("Missing subfunction code.  Choices are:\n");
      dump_subfunction_choices(3);
      exit(4);
      }

   for (i=0; subcmd_jump_tbl[i].subfunc_string != NULL; i++)
      {
      if (!strcmp(argv[1], subcmd_jump_tbl[i].subfunc_string))
         {
	 rv = (subcmd_jump_tbl[i].subfunc_ptr)(argc, argv);
         goto normal_return;
	 }
      }

   printf("Invalid subfunction code.  Choices are:\n");
   dump_subfunction_choices(3);
   exit(4);

   normal_return:
      exit(rv);
   }

//********************************************************************************
//$Log: arith_large_cgi.c,v $
//Revision 1.6  2003/07/01 03:46:58  dtashley
//Edits towards working continued fraction best rational approximation
//functionality.
//
//Revision 1.5  2003/06/29 22:51:49  dtashley
//Edits toward CF best approximation functionality.
//
//Revision 1.4  2003/04/17 20:02:05  dtashley
//License text for the GPL added.  All source files are now under the GPL,
//after some discussion on the GMP list.
//
//Revision 1.3  2003/04/15 23:55:43  dtashley
//SUBFUNC_PFACT_18 added.
//
//Revision 1.2  2003/04/14 23:27:01  dtashley
//Subfunction GMP_PROB_PRIME finished.
//
//Revision 1.1  2003/04/14 20:38:40  dtashley
//Initial checkin.
//********************************************************************************
//End of file ARITH_LARGE_CGI.C.
//********************************************************************************
